/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import * as grpc from '@grpc/grpc-js';
import * as protoLoader from '@grpc/proto-loader';
import { PeerCertificate } from 'tls';
export declare type gRPCRequest = {
    host: string;
    service: string;
    method: string;
    data?: object | object[];
    metadata?: gRPCRequestMetadata;
    tls?: gRPCRequestTLS;
    beforeRequest?: (req: gRPCRequest) => void;
    afterResponse?: (res: gRPCResponse) => void;
    loaderOptions?: protoLoader.Options;
    signal?: AbortSignal;
    options?: grpc.CallOptions;
};
export declare type gRPCRequestMetadata = {
    [key: string]: string | Buffer;
};
export declare type gRPCRequestTLS = {
    rootCerts?: string | Buffer;
    privateKey?: string | Buffer;
    certChain?: string | Buffer;
    verifyOptions?: VerifyOptions;
};
export declare type VerifyOptions = {
    checkServerIdentity?: CheckServerIdentityCallback;
};
export declare type CheckServerIdentityCallback = (hostname: string, cert: PeerCertificate) => Error | undefined;
export declare type gRPCResponse = {
    data: object | object[];
    size: number;
    statusCode?: number;
    statusMessage?: string;
    metadata?: {
        [key: string]: (string | Buffer)[];
    };
};
export declare function makeRequest(proto: string | string[], { beforeRequest, afterResponse, loaderOptions, options, ...clientConfig }: gRPCRequest): Promise<gRPCResponse>;
